const webpack = require('webpack');
const path = require('path');
const buildPath = path.resolve(__dirname, 'dist');
const nodeModulesPath = path.resolve(__dirname, 'node_modules');
const TransferWebpackPlugin = require('transfer-webpack-plugin');

const config = {
  entry: [
    'webpack/hot/dev-server',
    'webpack/hot/only-dev-server',
    path.join(__dirname, '/www/app.js'),
  ],
  resolve: {
    extensions: ["", ".js"]
  },
  devServer:{
    contentBase: '', 
    devtool: 'eval',
    hot: true,        
    inline: true,
    port: 3000,        
    host: 'localhost',
  },
  devtool: 'eval',
  output: {
    path: buildPath,    
    filename: 'app.js',
  },
  plugins: [
    new webpack.HotModuleReplacementPlugin(),
    new webpack.NoErrorsPlugin(),
    new TransferWebpackPlugin([
      {from: 'app'},
    ], path.resolve(__dirname, "dist")),
  ],
  module: {
    loaders: [
      {
        test: /\.js$/,  
        loaders: ['react-hot', 'babel'], 
        exclude: [nodeModulesPath],
      },
      {
        test: /\.(css|scss|sass)$/,
        loader: "style-loader!css-loader!sass-loader",
      }, 
      {
        test: /\.(png|jpg|gif|woff)$/,
        loader: 'url-loader?limit=8192'
      }, 
      {
        test: /\.(ttf|eot|svg)(\?v=[0-9]\.[0-9]\.[0-9])?$/,
        loader: "file-loader"
      }
    ],
  }
};

module.exports = config;
