const webpack = require('webpack');
const path = require('path');
const compressjs = new webpack.optimize.UglifyJsPlugin({compress: {warnings: false}});
const ExtractTextPlugin = require('extract-text-webpack-plugin');
const buildPath = path.resolve(__dirname, 'dist');
const ENV = process.env.NODE_ENV

const config = {
  entry: {
    index: './www/app.js',
  },
  output: {
    path: buildPath,
    filename: ENV === 'production' ? 'smartui.min.js' : 'smartui.js'
  },
  module: {
    loaders: [
      {
        test: /\.(css|scss)$/,
        loader: ExtractTextPlugin.extract("style-loader", "css-loader!sass-loader?minimize")
      }, {
        test: /\.js$/,
        exclude: /node_modules/,
        loader: "babel-loader"
      }, {
        test: /\.(png|jpg|gif|woff)$/,
        loader: 'url-loader?limit=8192'
      }, {
        test: /\.(ttf|eot|svg)(\?v=[0-9]\.[0-9]\.[0-9])?$/,
        loader: "file-loader"
      }
    ]
  },
  resolve: {
    root: path.resolve('./src'),
    extensions: ['', '.js', '.json', '.scss', '.png', 'jpg']
  },
  plugins: [
    new ExtractTextPlugin(ENV === 'production' ? 'smartui.min.css' : 'smartui.css', {allChunks: true})
  ]
};

if (ENV === 'production') {
  config.plugins.push(compressjs)
}

module.exports = config;